      program tom2gmt
* to convert solutions from tomographic inversions to
* input for Generic Mapping Tools.  
      include 'params.h'
*      parameter(halfpi=1.5707963267948966)
*      parameter(pi=3.1415926535897932)
*      parameter(twopi=6.2831853071795862)
*      parameter(radius=6371.)
*      parameter(pio180=0.017453293)
*      parameter(mxtr=800000)
*      parameter(mxmod=223)
*      parameter(ntri=3)
*      parameter(epsilon=0.0001)
*      dimension tt(mxtr),pp(mxtr)
*      dimension rr(mxmod)
*      common/npole/ola,olo,oazi
*      common/tnet/col,dmx,ntr,tt,pp,rr,nshells
*      common/tnetx/atmin,ddmn,fac,don,dan,edmx
      include 'partpwi.h'
*      parameter(nrfpr=5)
*      parameter(nlay=2)
*      parameter(nrntri=nlay*ntri)
*      parameter(nminv=1e7)
*      parameter(nbin=20)

      parameter(ncubelen=1000)
      parameter(extreme=99999.)
      dimension fq(nrntri),iq(nrntri)
      dimension ft(ntri),it(ntri)
      dimension fr(nlay),ir(nlay)
      dimension sol(nminv)
      dimension cols(nminv)
      dimension asol(ncubelen,ncubelen)
*      dimension a3d(nminv),c3d(nminv)
      dimension avas(223)
      dimension br(223),cr(223)
      dimension by(223,4),cy(223,4)
*      character*1 na3d(nminv),nc3d(nminv)
      dimension cs(ncubelen,ncubelen)
      character*15 txt(nrfpr)
      character*50 slicename,argv,mohoname,toponame,avername
      character*52 colname 
      character*2 aflg
      character*4 bname
      dimension jy(nrfpr)
      logical black,exag,back,relat,hte,aver,plbk
      data txt/'P-wave velocity','S-wave velocity','density',
     &         'Shear Quality','Moho depth'/

      write(6,*) 'program tom2gmt:'
10    format(a,$ )
20    format('#',1x,a16,1x,6f9.2)
30    format(i4,a17,5f8.2)
40    format(2f9.2)

      aver = .false.

      inquire(file='hiteps',exist=hte)
      if (hte) then
	 open(1,file='hiteps')
	 rewind 1
	 read(1,*) cseps
	 close(1)
	 write(6,*) 'Found ',cseps,' from file "hiteps"'
      else
         cseps = 0.012
	 write(6,*) 'Set white-out hitcnt value to ',cseps
	 write(6,*) '   value may be input through file "hiteps"' 
      endif

      inquire(file='topoexag',exist=exag)
      if (exag) then
        open(1,file='topoexag')
        read(1,*) topoexag
        close(1)
	write(6,*) 'Found ',topoexag,' from file "topoexag"'
      else 
        topoexag = 10.
	write(6,*) 'Set topography exaggeration to ',topoexag
	write(6,*) '   value may be input through file "topoexag"' 
      endif

      iar = iargc()
      relat=.false.
      back = .false.
      black = .false.
      plbk = .false.
      if (iar.eq.1) then
         call getarg(iar,argv)
         black = .false.
	 plbk = .false.
      else if (iar.gt.1) then
         do ia = 1,iar-1 
            call getarg(ia,aflg)
            if (aflg.eq.'-w') then
               inquire(file='columnsums',exist=black)
               if (.not.black) then
		  write(6,*) 'error: file columnsums does not exist'
	       endif
               write(6,*) 'eps (max blackout colsum)=',cseps
            else if (aflg.eq.'-a') then
               inquire(file='backgroundmodel',exist=back)
               if (.not.back) then
		  write(6,*) 'error: file backgroundmodel does not exist'
	       endif
            else if (aflg.eq.'-r') then
               inquire(file='backgroundmodel',exist=back)
               if (.not.back) then
		  write(6,*) 'error: file backgroundmodel does not exist'
	       endif
	       relat = .true.
               inquire(file='plotbackground',exist=plbk)
               if (plbk) then
		  write(6,*) 'Using different background for relative plotting!'
	       endif
            endif
         enddo
         call getarg(iar,argv)
      else
         write(6,*) 'Usage: tom2gmt [-w] [-a/-r] modelname'
         write(6,*) ''
         stop
      endif 
      if (black) then
         write(6,*) 'file columnsums used to white-out unsampled areas'
      endif
      if (back) then
         open(1,file='backgroundmodel',status='old')
         read(1,*) bname
         write(6,*) 'backgroundmodel used = ',bname
         read(1,*) 
         read(1,*) nba, nbi, nbo
         do i = 1,nbo 
	    read(1,*) 
	 enddo
         nba = nba - nbo
         do i = 1,nba 
            read(1,*) br(i),by(i,3),by(i,1),by(i,2),dum,by(i,4)
         enddo
         close(1)
* Get background Moho depth and convert to Moho radius in km:
	 call fzmoho(nba,br,by(1,1),zmoho)
	 rmoho = radius - 0.001*zmoho
         do i = 1,nba 
            br(i) = 0.001*br(i)
            by(i,1) = 0.001*by(i,1)
            by(i,2) = 0.001*by(i,2)
            by(i,3) = 0.001*by(i,3)
* work with 1/1000 of Qs (for consistency with output in km/s)
            by(i,4) = 0.001*by(i,4)
         enddo
      else
	 rmoho = radius - 30.
         write(6,*) "Not absolute: Assuming Moho is at 30 km"
      endif
      if (plbk) then
         open(1,file='plotbackground',status='old')
         read(1,*) bname
         write(6,*) 'plotbackground used = ',bname
         read(1,*) 
         read(1,*) nca, nci, nco
         do i = 1,nco 
	    read(1,*) 
	 enddo
         nca = nca - nco
         do i = 1,nca 
            read(1,*) cr(i),cy(i,3),cy(i,1),cy(i,2),dum,cy(i,4)
         enddo
         close(1)
* Get background Moho depth and convert to Moho radius in km:
	 call fzmoho(nca,cr,cy(1,1),zmoco)
	 rmoco = radius - 0.001*zmoco
         do i = 1,nca 
            cr(i) = 0.001*cr(i)
            cy(i,1) = 0.001*cy(i,1)
            cy(i,2) = 0.001*cy(i,2)
            cy(i,3) = 0.001*cy(i,3)
* work with 1/1000 of Qs (for consistency with output in km/s)
            cy(i,4) = 0.001*cy(i,4)
         enddo
      endif

      do i = 1,ncubelen 
         do j = 1,ncubelen 
            asol(i,j) = 0.
            cs(i,j) = 0.
         enddo
      enddo

      open(1,file='parcom',status='old')
      rewind 1
      read(1,*) njy
      read(1,*) (jy(j),j=1,njy)
      close(1)
      write(6,*) 'read parcom'
      do j=1,njy 
         if (jy(j).eq.5) then 
            jmoho = 1
         else
            jmoho = 0
         endif
      enddo

      call getgrid()
      nsq = nshells*ntr
      mnnn = ((njy-1)*nshells + jmoho)*ntr
      if (mnnn.gt.nminv) then
	 write(6,*) 'Error: dimension of solution ',mnnn,' > ',nminv
	 stop
      endif

      open(1,file=argv,status='old')
      rewind 1
      read(1,*) (dum,i=1,8),ndum
      read(1,*) (dum,i=1,ndum)
      read(1,*) (sol(i),i=1,mnnn)
      close(1)
      write(6,*) 'read ',argv

      if (black) then
         open(1,file='columnsums',status='old')
         rewind 1
	 do i=1,4
            read(1,*) 
	 enddo
         read(1,*) (cols(i),i=1,mnnn)
         close(1)
         write(6,*) 'read columnsums'
      endif

      jchoice = 0
      iopt = 0

      write(6,*) 'solution contains:'
      do j=1,njy 
	 write(6,*) j,' ',txt(jy(j)) 
      enddo
      do while (jchoice.lt.1.or.jchoice.gt.5) 
         write(6,10) 'which parameter (give number):'
         read(5,*) jchoice 
	 write(6,*) jchoice
      enddo
      ioff = (jchoice-1)*nsq 
      ioffm = (njy-1)*nsq
      kb = jy(jchoice)
      if (jy(jchoice).eq.5) then
         rshell = rmoho
         iopt = -3
         write(6,*) 'slice type, option -3'
      endif

      if (iopt.eq.0) then
         write(6,*) 'select slice type, options:'
         write(6,*) '1) latitude, longitude, azimuth, delta, 
     & depth-extent'
         write(6,*) '2) latitude1, longitude1, latitude2, longitude2, 
     & depth-extent'
         write(6,*) '3) depth = constant, latmin, latmax, lonmin, 
     & lonmax'
*         write(6,*) '4) brick as is (in cartesian grid)'
         write(6,*) '4) brick in lon, lat, r' 
         write(6,*) 'all in km or degrees'
         do while (iopt.lt.1.or.iopt.gt.4) 
            write(6,10) 'selected option: ' 
            read(5,*) iopt 
	 enddo
	 write(6,*) iopt
      endif

      write(6,10) "give filename for output: "
      read(5,'(a)') slicename
      write(6,*) slicename
      colname = 'cl'//slicename
      mohoname = 'mh'//slicename
      toponame = 'tp'//slicename
      avername = 'av'//slicename

* preparational:
      if (iopt.eq.1) then
         open(2,file=slicename)
         rewind 2
         if (black) then
            write(6,*) colname
            open(4,file=colname)
            rewind 4
         endif
         write(6,*) 'give latitude1, longitude1, azimuth, delta, 
     & depth-extent: '
         read(5,*) xlat1,xlon1,azimuth,delta,depth
         write(6,*) xlat1,xlon1,azimuth,delta,depth
         write(6,10) 'Give ddelta and ddepth:'
         read(5,*) da,db
         write(6,*) da,db
         xlat1 = xlat1*p
         xlon1 = xlon1*p
         delta = delta*p
         azimuth = azimuth*pio180
         call fiteta(fi2,teta2,xlon1,xlat1,azimuth,delta)
         xlon2 = fi2
         xlat2 = teta2
         write(6,*) 'To:',xlat2/pio180,xlon2/pio180
         call rotate(0,xlat1,xlon1,tetaout,fiout)
         xlat1 = tetaout
         xlon1 = fiout
         call rotate(0,xlat2,xlon2,tetaout,fiout)
         xlat2 = tetaout
         xlon2 = fiout
         call epdist(xlat1,xlon1,xlat2,xlon2,delta,azimuth)
         rup = radius
         rdown = radius - depth
      endif
      if (iopt.eq.2) then
         open(2,file=slicename)
         rewind 2
         if (black) then
            write(6,*) colname
            open(4,file=colname)
            rewind 4
         endif
         write(6,*) 'give latitude1, longitude1, latitude2, 
     & longitude2, depth-extent: '
         read(5,*) xlat1,xlon1,xlat2,xlon2,depth
         write(6,*) xlat1,xlon1,xlat2,xlon2,depth
         write(6,10) 'Give ddelta and ddepth:'
         read(5,*) da,db
         write(6,*) da,db
         xlat1 = xlat1*pio180
         xlat2 = xlat2*pio180
         xlon1 = xlon1*pio180
         xlon2 = xlon2*pio180
         call rotate(0,xlat1,xlon1,tetaout,fiout)
         xlat1 = tetaout
         xlon1 = fiout
         call rotate(0,xlat2,xlon2,tetaout,fiout)
         xlat2 = tetaout
         xlon2 = fiout
         call epdist(xlat1,xlon1,xlat2,xlon2,delta,azimuth)
         write(6,*) 'distance is ',delta/pio180,' degrees'
         rup = radius
         rdown = radius - depth
         iopt = 1
      endif
      if (iopt.eq.3) then
         open(2,file=slicename)
         rewind 2
         if (black) then
            write(6,*) colname
            open(4,file=colname)
            rewind 4
         endif
         write(6,*) 'give depth of layer, latmin, latmax, 
     &		     lonmin, lonmax: '
         read(5,*) rshell,atmin,atmax,onmin,onmax
         write(6,*) rshell,atmin,atmax,onmin,onmax
         write(6,10) 'Give dlat and dlon:'
         read(5,*) db,da
         write(6,*) db,da
         rshell = radius - rshell
      endif
      if (iopt.eq.-3) then
         open(2,file=slicename)
         rewind 2
         if (black) then
            write(6,*) colname
            open(4,file=colname)
            rewind 4
         endif
         write(6,*) 'give latmin, latmax, lonmin, lonmax: '
         read(5,*) atmin,atmax,onmin,onmax
         write(6,*) atmin,atmax,onmin,onmax
         write(6,10) 'Give dlat and dlon:'
         read(5,*) db,da
	 write(6,*) db,da
      endif
      if (iopt.eq.3.or.iopt.eq.-3) then 
         atmin = atmin*pio180
         onmin = onmin*pio180
         atmax = atmax*pio180
         onmax = onmax*pio180
         da = da*pio180
         db = db*pio180
         amin = onmin
         amax = onmax
         bmin = atmin
         bmax = atmax
         arange = amax-amin
         brange = bmax-bmin
         na = nint(arange/da) + 1
         nb = nint(brange/db) + 1
         if (nb.gt.ncubelen.or.na.gt.ncubelen) then
            write(6,*) "Dimension of asol exceeded!"
            stop
         endif
      endif
      if (iopt.eq.4.or.iopt.eq.5) then
         open(2,file=slicename)
         rewind 2
         if (black) then
            open(4,file=colname)
            rewind 4
         endif
      endif
      if (iopt.eq.5) then 
         write(6,*) 'give depthmin, depthmax, latmin, latmax, 
     &		     lonmin, lonmax: '
         read(5,*) depthmin,depthmax,atmin,atmax,onmin,onmax
         write(6,10) 'Give ddelta and ddepth:'
         read(5,*) da,db
         rup = radius - depthmin
         rdown = radius - depthmax
         range = rup - rdown
* two input increments: da,db. da = dlat = dlon and db = ddepth = dr
         nr = nint(range/db) + 1
         dr = db
         amin = atmin*pio180
         amax = atmax*pio180
         bmin = onmin*pio180
         bmax = onmax*pio180
         arange = amax-amin
         brange = bmax-bmin
         ddelta = ddelta/radius
         na = nint(arange/da) + 1
         nb = nint(brange/da) + 1
         db = da
      endif
* at this point all variables are in radians and km.
      cmax = 0.
      asmax = -1.*extreme
      asmin = extreme

* real work, construct "asol" for plot-file:
      if (iopt.eq.1) then 
         if (aver) then
	    open(20,file=avername)
	    rewind 20
         endif
         open(10,file=mohoname)
         rewind 10
         open(11,file=toponame)
         rewind 11
* we know: xlat1 xlon1 delta azimuth rup rdown xlat2 xlon2
         range = rup - rdown
         dr = db
         nb = nint(range/db) + 1
         da = da*pio180
         na = nint(delta/da) + 1
         if (nb.gt.ncubelen.or.na.gt.ncubelen) then
            write(6,*) "Dimension of asol exceeded!"
            stop
         endif
         write(6,*) "...slicing..."
         acoor = 90. + 0.5*delta/pio180
         dacoor = da/pio180
	 do j=1,nb
	    avas(j) = 0.
	 enddo
         do i = 1,na 
	    call gettsor(xlat1,xlon1,nt,it,ft)
            do j = 1,nb 
	       as = 0.
	       r = (rup - (j-1)*dr)
	       call getrf(r,nrl,ir,fr)
	       nq = 0
	       do lq =1,nrl
		  do kq = 1,nt
		     nq = nq+1
		     iq(nq) = ir(lq)+it(kq)
		     fq(nq) = fr(lq)*ft(kq)
		  enddo
	       enddo
               do l=1,nq 
                  as = as + fq(l)*sol(ioff+iq(l))
	       enddo
	       if (back) then
                  bas = finterp(r,br,by(1,kb),nba)
		  cas = bas
		  if (plbk) cas = finterp(r,cr,cy(1,kb),nca)
	          if (relat) then
	             as = 100.*(as+bas-cas)/cas
	          else
	             as = bas + as
                  endif
               endif
	       if (as.gt.asmax) then
	          asmax = as
	          elamax = acoor*pio180
	          elomax = r*pio180
	       endif
	       if (as.lt.asmin) then
	          asmin = as
	          elamin = acoor*pio180
	          elomin = r*pio180
	       endif
	       if (black) then
	          cst = 0.
	          do l=1,nq 
	             cst = cst + fq(l)*cols(ioff+iq(l))
	          enddo
	          if (abs(cst).lt.cseps) then 
	             as = extreme 
  	          else 
	             cmax = max(cmax,cst)
	          endif
                  cs(i,j) = cst
	       endif
               asol(i,j) = as
	       avas(j) = avas(j) + as
* Add 29 to make the earth's radius a multiple of 100 (6400),
* is necessary to get GMT polar projection to plot depth.
*               write(2,*) acoor,r+29.,as
               write(2,*) acoor,r+29.,as*1000.
               if (black) then
                  write(4,*) acoor,r+29.,cst
               endif
            enddo
* make line of Moho depth and topography:
            dmh = 0
*            call gett(out,col,xlat1,xlon1,ntr,tt,pp,ni,it,pt)
            do l=1,nt 
               dmh = dmh + ft(l)*sol(ioffm+it(l)) 
            enddo
*            dmh = rmoho + 29. - 0.001*dmh
            dmh = rmoho + 29. - dmh
            call rotate(1,xlat1,xlon1,rlat,rlon)
            drlat = rlat/pio180
            drlon = rlon/pio180
            topo = etopo5(drlat,drlon)
* scale topo so it's in km and (~10)x exaggerated:
            topo = radius + 29 + topoexag*0.001*topo
            acr = acoor
            if (i.eq.1) then
               acr = 0.9999*acoor
            endif 
            if(i.eq.na) then
               acr = 1.0001*acoor
            endif 
            write(11,40) acr, topo
            if (black) then 
               cstm = 0.
*               call gettlin(out,col,dmx,xlat1,xlon1,ntr,tt,pp,nil,itl,ptl)
	       do l=1,nt 
                  cstm = cstm + ft(l)*cols(ioffm+it(l))
	       enddo
               write(10,40) acr, dmh 
            else
               write(10,40) acr, dmh 
            endif
            call nextcoor(xlat1,xlon1,da,azimuth,xlat2,xlon2)
            acoor = acoor - dacoor
         enddo 
	 close(2)
	 if (aver) then
	    do j=1,nb
	       r = (rup - (j-1)*dr) + 29.
	       avas(j) = avas(j)/na
*	       write(20,*) r,avas(j)
	       write(20,*) r,1000.*avas(j)
	    enddo
	    close(20)
*	    open(2,file=slicename)
*	    rewind 2
*	    open(21,file='tmpslice')
*	    rewind 21
*	    do i=1,na
*	       do j=1,nb
*		  read(2,*) a,b,c
*		  write(21,*) a,b,c-avas(j)
*	       enddo
*	    enddo
*	    close(2)
*	    close(21)
	 endif
         write(6,*) 'written ',slicename
         write(6,*) 'nr of nodes: ',na,nb
         write(6,*) 'spacing (degrees,km): ',dacoor,dr
         close(10)
         close(11)
         write(6,*) 'written ',mohoname
         write(6,*) 'written ',toponame
         call mkclrpt(asmin,asmax)
      endif
      if (iopt.eq.3) then
         write(6,*) "...slicing..."
         if (back) then
            bas = finterp(rshell,br,by(1,kb),nba)
	    cas = bas
	    if (plbk) cas = finterp(rshell,cr,cy(1,kb),nca)
         endif
         ea = amin - da
	 call getrf(rshell,nrl,ir,fr)
         do i=1,na 
            ea = ea + da
            eb = bmax + db
            do j=1,nb 
	       as = 0.
               eb = eb - db
	       elo = ea 
	       ela = eb 
               call rotate(0,ela,elo,tetaout,fiout)
	       call gettsor(tetaout,fiout,nt,it,ft)
               nq = 0
               do lq = 1,nrl
                  do kq = 1,nt
                     nq = nq + 1
                     iq(nq) = ir(lq)+it(kq)
                     fq(nq) = fr(lq)*ft(kq)
                  enddo
               enddo
               do l=1,nq 
                  as = as + fq(l)*sol(ioff+iq(l))
	       enddo
	       if (back) then
	          if (relat) then
*	             as =  0.1*as/bas
*	             as =  100.*as/bas
	             as =  100.*(as+bas-cas)/cas
	          else
*	             as = bas + 0.001*as
	             as = bas + as
	          endif
	       endif
	       if (as.gt.asmax) then
	          asmax = as
	          elamax = ela
	          elomax = elo
	       endif
	       if (as.lt.asmin) then
	          asmin = as
	          elamin = ela
	          elomin = elo
	       endif
	       if (black) then
	          cst = 0.
*                  call getclin(x,y,z,nql,iql,pql,out)
	          do l=1,nq 
	             cst = cst + fq(l)*cols(ioff+iq(l))
	          enddo
	          if (abs(cst).lt.cseps) then 
	             as = extreme 
  	          else 
	             cmax = max(cmax,cst)
	          endif
                  cs(i,j) = cst
	       endif
               asol(i,j) = as
*               write(2,*) elo/pio180,ela/pio180,as
               write(2,*) elo/pio180,ela/pio180,as*1000.
            enddo
         enddo
         write(6,*) 'written ',slicename
         write(6,*) 'nr of nodes: ',na,nb
         write(6,*) 'spacing (degrees): ',da/pio180,db/pio180
         call mkclrpt(asmin,asmax)
      endif
      if (iopt.eq.-3) then 
*  this is the option for plotting crustal thickness
         write(6,*) "...slicing..."
         ea = amin - da
         do i=1,na 
            ea = ea + da
            eb = bmax + db
            do j=1,nb 
	       as = 0.
               eb = eb - db
	       elo = ea 
	       ela = eb 
               call rotate(0,ela,elo,tetaout,fiout)
               call gettsor(tetaout,fiout,nt,it,ft)
*               call gett(out,col,tetaout,fiout,ntr,tt,pp,ni,it,pt)
	       do l=1,nt 
                  as = as + ft(l)*sol(ioffm+it(l)) 
	       enddo
*        get the output in km rather than m of Moho anomaly:
*               as = 0.001*as
	       if (back) then
	          as = as + radius - rmoho
	       endif
	       if (as.gt.asmax) then
	          asmax = as
	          elamax = ela
	          elomax = elo
	       endif
	       if (as.lt.asmin) then
	          asmin = as
	          elamin = ela
	          elomin = elo
	       endif
	       if (black) then
	          cst = 0.
*                  call gettlin(out,col,dmx,tetaout,fiout,ntr,tt,pp,nil,itl,ptl)
	          do l=1,nt 
	             cst = cst + ft(l)*cols(ioffm+it(l))
	          enddo
	          if (abs(cst).lt.cseps) then 
	             as = extreme 
  	          else
	             cmax = max(cmax,cst)
	          endif
                  cs(i,j) = cst
	       endif
               asol(i,j) = as
*               write(2,*) elo/pio180,ela/pio180,as
               write(2,*) elo/pio180,ela/pio180,as*1000.
               if (black) then
                  write(4,*) elo/pio180,ela/pio180,cst
               endif
            enddo
         enddo
         write(6,*) 'written ',slicename
         write(6,*) 'nr of nodes: ',na,nb
         write(6,*) 'spacing (degrees): ',da/pio180,db/pio180
*         open(8,file='spacing')
*         write(8,'(f9.5,"/",f9.5)') da/p,db/p
*         close(8)
         call mkclrpt(asmin,asmax)
      endif
      if (iopt.eq.4) then
         write(6,*) "rendering..."
         ipatient = 1
         ijk = 0
         r = rdown - dr
         do k = 1,nr 
            r = r + dr
            call getrf(r,nrl,ir,fr)
            a = amin - da
            do j = 1,na 
               a = a + da
               b = bmin - db
               do i = 1,nb 
                  b = b + db
                  ijk = ijk + 1
                  as = 0
                  if (ijk.gt.ipatient*50000) then
                     ipatient = ipatient + 1
                     write(6,*) 'processed 50000 more knots'
                  endif
	          call rotate(0,a,b,teta,fi)
                  call gettsor(teta,fi,nt,it,ft)
                  nq = 0
                  do lq = 1,nrl
                     do kq = 1,nt
                        nq = nq + 1
                        iq(nq) = ir(lq)+it(kq)
                        fq(nq) = fr(lq)*ft(kq)
                     enddo
                  enddo
*                  call rtfxyz(0,teta,fi,r,x,y,z)
*                  call getc(x,y,z,r,teta,fi,nq,iq,pq,out)
                  do l=1,nq 
                     as = as + fq(l)*sol(ioff + iq(l))
                  enddo
	          if (back) then
                     bas = finterp(r,br,by(1,kb),nba)
	             if (relat) then
*	                as = 0.1*as/bas
	                as = 100.*as/bas
	             else
*	                as = bas + 0.001*as
	                as = bas + as
	             endif
	          endif
	          if (as.gt.asmax) then
	             asmax = as
	             elamax = teta
	             elomax = fi
	          endif
	          if (as.lt.asmin) then
	             asmin = as
	             elamin = teta
	             elomin = fi
	          endif
                  if (black) then
                     cst = 0.
*                     call getclin(x,y,z,nql,iql,pql,out)
                     do l=1,nq 
                        cst = cst + fq(l)*cols(ioff+iq(l))
                     enddo
                     if (abs(cst).lt.cseps) then
                        as = extreme
                     else
                        cmax = max(cmax,cst)
                     endif
*                     c3d(ijk) = cst
                  endif
*                  a3d(ijk) = as
               enddo
            enddo
         enddo
         gmax = max(asmax,-1.*asmin)
         factor = 127.5/gmax
         do ijk = 1,nsq 
*            ntmp = nint(127.5+a3d(ijk)*factor)
*            ntmp = max(ntmp,0)
*            ntmp = min(ntmp,255)
*            na3d(ijk) = char(ntmp)
         enddo
         do ijk = 1,nsq 
*            ierr = fputc(2,na3d(ijk))
         enddo
         open(9,file='Bobfile')
         rewind 9
         write(9,*) "size = ",nb,na,nr
         write(9,*) "file = ",slicename
         write(9,*) "color = rainbow.palette"
         close(9)
         if (black) then
            factor = 254./cmax
            do ijk = 1,nsq 
*               nc3d(ijk) = char(nint(c3d(ijk)*factor))
            enddo
            do ijk = 1,nsq 
*               ierr = fputc(4,nc3d(ijk))
            enddo
         endif
      endif
   
      if (black) then
         write(6,*) 'colsm max: ',cmax 
      endif
      write(6,*) 'asmin, asmax: ',asmin,asmax
      write(6,*) 'at:',elamin/pio180,elomin/pio180,' 
     &            and ',elamax/pio180,elomax/pio180

      write(6,*) 'tom2gmt finished'
      end
